package examples.hibernate.domain;

import javax.persistence.*;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

@Entity
public class Airline {
    /** 
     * Unique identifier.
     */
    @SuppressWarnings("unused")
    @Id @GeneratedValue(strategy=GenerationType.AUTO)
    private Long id;

    /**
     * Object version number.  
     * Necessary for the optimistic policy for concurrency control
     */
    @SuppressWarnings("unused")
    @Version
    private Long objVersion;    

    private String name;

    @OneToMany(mappedBy="airline")
    private Set<Airplane> airplanes;       // one-to-many bidirectional association

    protected Airline() { }
    
    public Airline(String name) {
	airplanes = new HashSet<Airplane>();
	setName(name);
    }
        
    public String getName() {
	return name;
    }

    public void setName(String name) {
	this.name = name;
    }

    public Set<Airplane> getAirplanes() {
	return Collections.unmodifiableSet(airplanes);
    }
    
    public void addAirplane(Airplane airplane) {
	this.airplanes.add(airplane);
	airplane.setAirline(this);
    }

    public void removeAirplane(Airplane airplane) {
	this.airplanes.remove(airplane);
	airplane.setAirline(null);
    }
    
    public String toString() {
	StringBuilder sb = new StringBuilder("Airline: ").append(name);
	if (!airplanes.isEmpty()) {
	    sb.append("\nFleet:");
	    for (Airplane airplane : airplanes) {
		sb.append("\n").append(airplane).append("\n\t*****");
	    }
	}
	
	return sb.toString();
    }
}
