package examples.hibernate.domain;

import javax.persistence.*;

import java.text.SimpleDateFormat;
import java.util.Date;


@Entity
public class Airplane extends Aircraft {
    public enum AeroplaneState { MAINTENANCE_DUE, FLYING, GROUNDED }

    private static int aeroplaneCount;       // transient property
    private transient double weightInPounds; // transient property

    @Transient
    private String currentPilot;             // transient property
    
    private long wingSpan;                   // persistent property
    private long length;                     // persistent property
    
    @Enumerated(EnumType.STRING)
    private AeroplaneState state;            // enum persisted as String in database
    
    @Temporal(TemporalType.DATE)
    private Date lastMaintenance;            // persistent property
    
    @Lob
    private String specs;                    // persistent property persisted as a CLOB

    @ManyToOne
    private Airline airline;                 // many-to-one bidirectional association (owner)

    protected Airplane() { }
    
    public Airplane(String name, long weigth, long wingSpan, long length) {
	super(name, weigth);
	aeroplaneCount++;
	setWingSpan(wingSpan);
	setLength(length);
	setState(AeroplaneState.FLYING);
	setLastMaintenance(new Date());
	setAirline(null);
    }

    public static long getAeroplaneCount() {
	return aeroplaneCount;
    }

    @Override
    public void setWeight(long weight) {
	super.setWeight(weight);
	weightInPounds = weight * 2.20462262185;
    }

    public double getWeightInPounds() {
	return weightInPounds;
    }

    public String getCurrentPilot() {
	return currentPilot;
    }
    
    public void setCurrentPilot(String currentPilot) {
	this.currentPilot = currentPilot;
    }

    public Date getLastMaintenance() {
	return new Date(lastMaintenance.getTime());
    }
    
    public void setLastMaintenance(Date date) {
	this.lastMaintenance = date;
    }

    public void resetLastMaintenance() {
	this.lastMaintenance = new Date();
    }

    public AeroplaneState getState() {
	return state;
    }

    public void setState(AeroplaneState state) {
	this.state = state;
    }

    public long getLength() {
	return length;
    }

    public void setLength(long length) {
	this.length = length;
    }
    
    public long getWingSpan() {
	return wingSpan;
    }

    public void setWingSpan(long wingSpan) {
	this.wingSpan = wingSpan;
    }

    public String getSpecs() {
	return specs;
    }
    
    public void setSpecs(String specs) {
	this.specs = specs;
    }

    public Airline getAirline() {
	return airline;
    }

    protected void setAirline(Airline airline) {
	this.airline = airline;
    }

    @Override
    public String toString() {
	StringBuilder sb = new StringBuilder(super.toString()).append("\n\tLength:")
		.append(length).append("m")
		.append("\n\tWing span: ").append(wingSpan).append("m")
		.append("\n\tSpecifications: ")
		.append(specs)
		.append("\n\tMaintenance State: ")
		.append(state)
		.append("\n\tLast Maintenance: ")
		.append(SimpleDateFormat.getDateInstance().format(lastMaintenance));

	return sb.toString();
    }

}
